<?php

/**
 * $Id: class.PathInformation.php,v 1.6 2005/03/16 21:47:46 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     core
 * @subpackage  class
 * @access      public
 *
 * @author      Niels Braczek <nbraczek@bsds.de>
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.6 $
 */

/**
 * coWiki - Path Information class
 *
 * @package     core
 * @subpackage  class
 * @access      public
 *
 * @author      Niels Braczek <nbraczek@bsds.de>
 * @since       coWiki 0.4.0
 */
class PathInformation extends Object {

    protected static $Instance = null;

    // --------------------------------------------------------------------

    /**
     * Get instance
     *
     * @access  public
     * @return  Singleton
     *
     * @author  Niels Braczek <nbraczek@bsds.de>
     * @since   coWiki 0.4.0
     */
    public function getInstance() {
        if (!self::$Instance) {
            self::$Instance = new PathInformation;
        }
        return self::$Instance;
    }

    // --------------------------------------------------------------------

    /**
     * Class constructor
     *
     * @access  protected
     * @return  void
     *
     * @author  Niels Braczek <nbraczek@bsds.de>
     * @since   coWiki 0.4.0
     *
     * @throws  ConfigNotReadableException
     * @throws  ConfigSyntaxException
     */
    protected function __construct() {

        /** Get relative value */
        define('COWIKI_RELATIVE', 0);
        /** Get absolute value */
        define('COWIKI_ABSOLUTE', 1);

        // Set default directory separator
        if (!defined('DIRECTORY_SEPARATOR')) {
            define('DIRECTORY_SEPARATOR', '/');
        }

        // ----------------------------------------------------------------

        // Absolute fs path to coWiki include directory
        $sIncludePath = realpath(dirname(__FILE__) . '/../..');
        $sIncludePath = $this->unifySlashes($sIncludePath);
        $this->set('PATH_INCLUDE', $sIncludePath);

        // Absolute fs path to coWiki home directory
        $sFsBasePath = dirname($_SERVER['PATH_TRANSLATED']);
        $sFsBasePath = $this->unifySlashes($sFsBasePath);
        $this->set('PATH_BASE', $sFsBasePath);

        // Relative path to coWiki home directory seen from document root
        $sCoWikiRoot = dirname($_SERVER['PHP_SELF']);
        $sCoWikiRoot = $this->unifySlashes($sCoWikiRoot, '/');
        $this->set('PATH_COWIKI', $this->removeLeadingSlash($sCoWikiRoot));

        // ----------------------------------------------------------------

        // Absolute URI of coWiki home directory
        $sHttpBasePath = 'http://' . $_SERVER['HTTP_HOST'] . $sCoWikiRoot;
        $sHttpBasePath = $this->unifySlashes($sHttpBasePath, '/');
        $this->set('HTTP_BASE_URI', $sHttpBasePath);

        // ----------------------------------------------------------------

        // Subdirectories
        $this->set('PATH_IMAGES', 'img');
        $this->set('PATH_TEMPLATE', 'tpl');

        // ----------------------------------------------------------------

        // Pathes defined in core.conf
        try {
            $sPath = $sIncludePath . '/core.conf';

            $Conf = new Config($sPath);
            $aConf = $Conf->getConfigAsArray();

        } catch (ConfigNotReadableException $cnre) {
            throw $cnre; // rethrow
        } catch (ConfigSyntaxException $cse) {
            throw $cse; // rethrow
        }

        if (isset($aConf['PATH_TEMP'])) {
            $this->set(
                'PATH_TEMP',
                $this->cleanConfPath($aConf['PATH_TEMP'])
            );
        }
        if (isset($aConf['PATH_BINARY'])) {
            $this->set(
                'PATH_BINARY',
                $this->cleanConfPath($aConf['PATH_BINARY'])
            );
        }
        if (isset($aConf['PATH_DIFF'])) {
            $sExecutable = basename($aConf['PATH_DIFF']);
            $sDiffPath   = $this->cleanConfPath(dirname($aConf['PATH_DIFF']))
                           . DIRECTORY_SEPARATOR . $sExecutable;
            if ($sDiffPath == DIRECTORY_SEPARATOR) {
                $sDiffPath = '';
            }
            $this->set('PATH_DIFF', $sDiffPath);
        }

        // ----------------------------------------------------------------

        // Set templates
        if (isset($aConf['RUNTIME_TEMPLATE_ACTIVE'])) {
            $sTemplate = $aConf['RUNTIME_TEMPLATE_ACTIVE'];
        } else {
            $sTemplate = 'default';
        }
        $this->setTemplate($sTemplate);
        // Fallback
        $this->setdefaultTemplate();
    }

    // --------------------------------------------------------------------

    /**
     * Clean path values from config file.
     *
     * @access  private
     * @param   string   The Path from config file
     * @return  string   The absolute path
     *
     * @author      Niels Braczek <nbraczek@bsds.de>
     * @since       coWiki 0.4.0
     */
    private function cleanConfPath($sPath) {

        $sPath = $this->unifySlashes($sPath, '/');

        if ($sPath == '') {
            return '';
        }

        // A path does *not* end with a slash
        if (substr($sPath, -1) == '/') {
            $sPath = substr($sPath, 0, -1);
        }

        if (strlen($sPath) > 1) {
            if ($sPath{0} != '/' && $sPath{1} != ':') {
                // path starts in coWiki Root
                $sPath = $this->get('PATH_BASE') .
                         '/' . $sPath;
            }
        }

        return $this->unifySlashes(realpath($sPath));
    }

    // --------------------------------------------------------------------

    /**
     * Remove leading slash.
     *
     * @access  private
     * @param   string   The path
     * @return  string   Path without leading slash
     *
     * @author      Niels Braczek <nbraczek@bsds.de>
     * @since       coWiki 0.4.0
     */
    private function removeLeadingSlash($sPath) {

        if ($sPath == '') {
            return $sPath;
        }
        if ($sPath{0} == '/') {
            $sPath = substr($sPath, 1);
        }
        return $sPath;
    }
    // --------------------------------------------------------------------

    /**
     * Set the template paths.
     *
     * The path variables
     * - PATH_ACT_TEMPLATE
     * - PATH_ACT_TEMPLATE_IMAGES
     * are set according to the specified template.
     *
     * @access  public
     * @param   string   The template name
     * @return  void
     *
     * @author      Niels Braczek <nbraczek@bsds.de>
     * @since       coWiki 0.4.0
     */
    public function setTemplate($sTemplate) {

        // Path to active template
        $sActTpl = $this->get('PATH_TEMPLATE') . '/' . $sTemplate;
        $this->set('PATH_ACT_TEMPLATE', $sActTpl);

        // Path to active images
        $sActImg = $sActTpl . '/'. $this->get('PATH_IMAGES');
        $this->set('PATH_ACT_TEMPLATE_IMAGES', $sActImg);
    }

    // --------------------------------------------------------------------

    /**
     * Set the default template paths.
     *
     * The path variables
     * - PATH_DEF_TEMPLATE
     * - PATH_DEF_TEMPLATE_IMAGES
     * are set to the specified default template. If no tmaplate name is
     * specified, 'default' is assumed.
     *
     * @access  public
     * @param   string   (optional) The default template name
     * @return  void
     *
     * @author      Niels Braczek <nbraczek@bsds.de>
     * @since       coWiki 0.4.0
     */
    public function setdefaultTemplate($sTemplate='default') {

        // Path to active template
        $sDefTpl = $this->get('PATH_TEMPLATE') . '/' . $sTemplate;
        $this->set('PATH_DEF_TEMPLATE', $sDefTpl);

        // Path to active images
        $sDefImg = $sDefTpl . '/'. $this->get('PATH_IMAGES');
        $this->set('PATH_DEF_TEMPLATE_IMAGES', $sDefImg);
    }


    // --------------------------------------------------------------------

    /**
     * Unify the directory separator.
     *
     * @access  private
     * @param   string   The path to be unified
     * @param   string   (optional) Directory separator
     * @return  string   The unified path
     *
     * @author      Niels Braczek <nbraczek@bsds.de>
     * @since       coWiki 0.4.0
     */
    private function unifySlashes($sPath, $sSlash=null) {

        if ($sSlash === null) {
            $sSlash = DIRECTORY_SEPARATOR;
        }
        $sPath = str_replace('\\', $sSlash, $sPath);
        $sPath = str_replace('/',  $sSlash, $sPath);
        // remove trailing slash
        $sPath = preg_replace('~' . $sSlash . '$~', '', $sPath);

        if ($sPath == $sSlash) {
            $sPath = '';
        }
        return $sPath;
    }

    // --------------------------------------------------------------------

    /**
     * Get a path.
     *
     * Returns a relative or absolute path (depending on the setting of
     * the second parameter), specified by a path identifier. Valid
     * values for the path identifier are
     * - BASE                     base path, i.e. coWiki root
     * - BINARY                   path to binary files
     * - CLASS                    class include directory
     * - CONFIG                   directory containing core.conf
     * - DIFF                     path to the diff utility
     * - IMAGES                   coWiki image directory
     * - INCLUDE                  cowiki include directory (same as CONFIG)
     * - LOCALE                   locale include directory
     * - PLUGIN                   plugin include directory
     * - TEMP                     path to temporary files
     * - TEMPLATE                 basic template directory
     * - TEMPLATE_ACTIVE          current template directory
     * - TEMPLATE_ACTIVE_IMAGES   image directory of current template
     * - TEMPLATE_DEFAULT         default template directory
     * - TEMPLATE_DEFAULT_IMAGES  image directory of default template
     *
     * @access  public
     * @param   string   The path identifier
     * @param   string   (optional) Directory separator
     * @return  string   The requested path
     *
     * @author  Niels Braczek <nbraczek@bsds.de>
     * @since   coWiki 0.4.0
     *
     * @todo    Don't use GenericException
     * @throws  GenericException
     */
    public function getPath($sPathVar, $nMode=COWIKI_ABSOLUTE) {

        // Set absolute part, if requested
        switch ($nMode) {

            case COWIKI_RELATIVE:
                $sPath = '';
                break;

            case COWIKI_ABSOLUTE:
                $sPath = $this->get('PATH_BASE') . '/';
                break;

            default:
                throw new GenericException('Illegal value in mode parameter');
                break;
        }

        // ----------------------------------------------------------------

        switch ($sPathVar) {

            case 'CONFIG':
            case 'INCLUDE': // always absolute
                $sPath = $this->get('PATH_INCLUDE');
                break;
            case 'BASE':    // always absolute
                $sPath = $this->get('PATH_BASE');
                break;

            case 'CLASS':   // always absolute
                $sPath = $this->get('PATH_INCLUDE') . '/class';
                break;
            case 'LOCALE':  // always absolute
                $sPath = $this->get('PATH_INCLUDE') . '/locale';
                break;
            case 'PLUGIN':  // always absolute
                $sPath = $this->get('PATH_INCLUDE') . '/plugin';
                break;

            case 'TEMP':    // always absolute
                $sPath = $this->get('PATH_TEMP');
                break;
            case 'BINARY':  // always absolute
                $sPath = $this->get('PATH_BINARY');
                break;
            case 'DIFF':    // always absolute
                $sPath = $this->get('PATH_DIFF');
                break;

            case 'TEMPLATE':
                $sPath = $sPath . $this->get('PATH_TEMPLATE');
                break;
            case 'TEMPLATE_ACTIVE':
                $sPath = $sPath . $this->get('PATH_ACT_TEMPLATE');
                break;
            case 'TEMPLATE_DEFAULT':
                $sPath = $sPath . $this->get('PATH_DEF_TEMPLATE');
                break;

            case 'IMAGES':
                $sPath = $sPath . $this->get('PATH_IMAGES');
                break;
            case 'TEMPLATE_ACTIVE_IMAGES':
                $sPath = $sPath . $this->get('PATH_ACT_TEMPLATE_IMAGES');
                break;
            case 'TEMPLATE_DEFAULT_IMAGES':
                $sPath = $sPath . $this->get('PATH_DEF_TEMPLATE_IMAGES');
                break;

            default:
                throw new GenericException(
                        'Illegal path identifier ' . $sPathVar
                      );
                break;
        }

        return $this->unifySlashes($sPath);
    }

    // --------------------------------------------------------------------

    /**
     * Get an URI.
     *
     * Returns an absolute URI specified by an URI identifier. Valid
     * values for the URI identifier are
     * - BASE                     base path, i.e. coWiki root
     * - IMAGES                   coWiki image directory
     * - TEMPLATE                 basic template directory
     * - TEMPLATE_ACTIVE          current template directory
     * - TEMPLATE_ACTIVE_IMAGES   image directory of current template
     * - TEMPLATE_DEFAULT         default template directory
     * - TEMPLATE_DEFAULT_IMAGES  image directory of default template
     *
     * @access  public
     * @param   string   The URI identifier
     * @return  string   The requested URI
     *
     * @author  Niels Braczek <nbraczek@bsds.de>
     * @since   coWiki 0.4.0
     *
     * @todo    Don't use GenericException
     * @throws  GenericException
     */
    public function getUri($sUriVar, $nMode=COWIKI_ABSOLUTE) {

        // Set absolute part, if requested
        switch ($nMode) {

            case COWIKI_RELATIVE:
                $sUri = $this->get('PATH_COWIKI') .'/';
                break;

            case COWIKI_ABSOLUTE:
                $sUri = $this->get('HTTP_BASE') .'/';
                break;

            default:
                throw new GenericException('Illegal value in mode parameter');
                break;
        }
        if ($sUri == '/') {
            $sUri = '';
        }

        // ----------------------------------------------------------------

        switch ($sUriVar) {

            case 'BASE':
                // already set
                break;

            case 'TEMPLATE':
                $sUri = $sUri . $this->get('PATH_TEMPLATE');
                break;
            case 'TEMPLATE_ACTIVE':
                $sUri = $sUri . $this->get('PATH_ACT_TEMPLATE');
                break;
            case 'TEMPLATE_DEFAULT':
                $sUri = $sUri . $this->get('PATH_DEF_TEMPLATE');
                break;

            case 'IMAGES':
                $sUri = $sUri . $this->get('PATH_IMAGES');
                break;
            case 'TEMPLATE_ACTIVE_IMAGES':
                $sUri = $sUri . $this->get('PATH_ACT_TEMPLATE_IMAGES');
                break;
            case 'TEMPLATE_DEFAULT_IMAGES':
                $sUri = $sUri . $this->get('PATH_DEF_TEMPLATE_IMAGES');
                break;

            default:
                throw new GenericException(
                        'Illegal URI identifier ' . $sUriVar
                      );
                break;
        }

        return $sUri;
    }

} // of class

?>