<?php
/**
 *
 * $Id: class.ParserTest.php,v 1.9 2006/05/07 14:03:57 cmarble Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * <pre>
 * Helping hands: Archie Campbell <cephalicmarble@blueyonder.co.uk>
 * </pre>
 *
 * @author      Paul Hanchett <paulha@aracnet.com>
 * @copyright   (C) Paul Hanchett, 2005
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.9 $
 *
 */

//--------------------------------------------------------------------------
//      Basic tests for formatting paragraphs
//--------------------------------------------------------------------------
class ParserBasicFormattingTest extends PHPUnit_TestCase
{
    var $p = NULL;
    var $r = NULL;
    var $c = NULL;
    //--------------------------------------------------------------
    //      Setup: Set up an instance of the parser for testing
    //--------------------------------------------------------------
    function setUp()
    {
        $this->p = TestWikiParser :: getInstance();
        $this->r = TestReverseParser :: getInstance();
        $this->c = WikiPseudoXmlConverter :: getInstance();
    }

    //--------------------------------------------------------------
    //  Test an empty string
    //--------------------------------------------------------------
    function testEmptyString()
    {
        $inputString = "";
        $outputString = "<document></document>";
        $messageString = "Empty input should produce empty output";
        $reformString = "";
        $ressageString = "Empty output should produce empty input"; 
        $this->assertEquals($outputString, $this->p->parse($inputString), $messageString);
        $this->assertEquals($reformString, $this->r->parse($outputString), $ressageString);
    }

    //--------------------------------------------------------------
    //  Test blank lines
    //--------------------------------------------------------------
    function testBlankLines()
    {
        $inputString = "\n\n\r\n";
        $outputString = "<document><![CDATA[\n]]><![CDATA[\n]]><![CDATA[\n]]></document>";
        $reverseString = "\n\n\n";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($reverseString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test line with no terminator
    //--------------------------------------------------------------
    function testShortLine()
    {
        $inputString = "Line with no eol";
        $outputString = "<document><![CDATA[Line with no eol]]></document>";
        $this->assertEquals($this->p->parse($inputString),$outputString);
        $this->assertEquals($this->r->parse($outputString),$inputString);
    }

    //--------------------------------------------------------------
    //  Test a single plain paragraph
    //--------------------------------------------------------------
    function testParagraphLine()
    {
        $inputString = "Line with eol\n";
        $outputString = "<document><![CDATA[Line with eol\n]]></document>";
        $this->assertEquals($this->p->parse($inputString),$outputString);
        $this->assertEquals($this->r->parse($outputString),$inputString);
    }

    //--------------------------------------------------------------
    //  Test two paragraphs separated by <eoln>
    //--------------------------------------------------------------
    function testTwoParagraphs()
    {
        $inputString = "<p>Line with eol</p>\n<p>And another paragraph</p>";
        $outputString = "<document><p><![CDATA[Line with eol]]></p><![CDATA[\n]]><p><![CDATA[And another paragraph]]></p></document>";
        $this->assertEquals($this->p->parse($inputString),$outputString);
        $this->assertEquals($this->r->parse($outputString),$inputString);
    }

    //--------------------------------------------------------------
    //  Test paragraphs are recognized with <cr> only eoln
    //--------------------------------------------------------------
    function testCROnlyLineEndings()
    {
        $inputString = "A\rB\rC\r";
        $outputString = "<document><![CDATA[A\n]]><![CDATA[B\n]]><![CDATA[C\n]]></document>";
        $reverseString = "A\nB\nC\n";
        $this->assertEquals($this->p->parse($inputString),$outputString);
        $this->assertEquals($this->r->parse($outputString),$reverseString);
    }

    //--------------------------------------------------------------
    //  Test paragraphs are recognized with <cr><lf> eoln
    //--------------------------------------------------------------
    function testCRLFLineEndings()
    {
        $inputString = "A\r\nB\r\nC\r\n";
        $outputString = "<document><![CDATA[A\n]]><![CDATA[B\n]]><![CDATA[C\n]]></document>";
        $reverseString = "A\nB\nC\n";
        $this->assertEquals($this->p->parse($inputString),$outputString);
        $this->assertEquals($this->r->parse($outputString),$reverseString);
    }

    //--------------------------------------------------------------
    //  Test paragraphs are recognized with <lf><cr> eoln
    //--------------------------------------------------------------
    function testLFCRLineEndings()
    {
        $inputString = "A\n\rB\n\rC\n\r";
        $outputString = "<document><![CDATA[A\n]]><![CDATA[B\n]]><![CDATA[C\n]]></document>";
        $reverseString = "A\nB\nC\n";
        $this->assertEquals($this->p->parse($inputString),$outputString);
        $this->assertEquals($this->r->parse($outputString),$reverseString);
    }
}

//--------------------------------------------------------------------------
//      Basic tests for formatting text within paragraphs
//--------------------------------------------------------------------------
class ParserEmphasisFormattingTest extends PHPUnit_TestCase
{
    var $p = NULL;
    var $r = NULL;

    var $testType = "Undefined";
    var $formatChar = "?";
    var $startTag = "<start>";
    var $endTag = "</start>";

    //--------------------------------------------------------------
    //  Standard constructor for unit tests.
    //--------------------------------------------------------------
    function __construct($name, $type, $format, $start, $end)
    {
        $this->formatChar = $format;
        $this->startTag = $start;
        $this->endTag = $end;
        $this->PHPUnit_TestCase($name);
    }

    //--------------------------------------------------------------
    //  Setup: Set up an instance of the parser for testing
    //--------------------------------------------------------------
    function setUp()
    {
        $this->p = TestWikiParser :: getInstance();
        $this->r = TestReverseParser :: getInstance();
    }

    //--------------------------------------------------------------
    //  Helper function to generate input and output strings 
    //--------------------------------------------------------------
    function generateWikiEmphasis($emphasis, $startTag, $endTag, $begin, $middle, $ending, & $input, & $output)
    {
        $input = $begin.$emphasis.$middle.$emphasis.$ending;
        $output = "<document>".($begin!=''?"<![CDATA[".$begin."]]>":"").$startTag."<![CDATA[".$middle."]]>".$endTag.($ending!=''?"<![CDATA[".$ending."]]>":"")."</document>";
    }

    //--------------------------------------------------------------
    //  TODO:   Need to check how special characters like punctuation
    //                  affect emphasis.  "This *word!* should /emphasize/.
    //--------------------------------------------------------------

    //--------------------------------------------------------------
    //  Test emphasis on a single word
    //--------------------------------------------------------------
    function testEmphasizeWord()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiEmphasis($this->formatChar.$this->formatChar, $this->startTag, $this->endTag, "start ", "middle", " end", $inputString, $outputString);
        
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test emphasis on a phrase
    //--------------------------------------------------------------
    function testEmphasizePhrase()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiEmphasis($this->formatChar.$this->formatChar, $this->startTag, $this->endTag, "start ", "middle words", " end", $inputString, $outputString);
        
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test emphasize an entire line (no terminator)
    //--------------------------------------------------------------
    function testEmphasizeLine()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiEmphasis($this->formatChar.$this->formatChar, $this->startTag, $this->endTag, "", "middle words", "", $inputString, $outputString);
        
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test mismatched emphasis
    //--------------------------------------------------------------
    function testMismatchedEmphasizedPhrase()
    {
        $inputString = "this phrase ".$this->formatChar.$this->formatChar."is mismatched";

        $outputString = "<document><![CDATA[this phrase ]]>"."<![CDATA[".$this->formatChar.$this->formatChar."]]>"."<![CDATA[is mismatched]]>"."</document>";

        
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test "this phrase **is** **mismatched in an interesting way"
    //--------------------------------------------------------------
    function testMismatchedEmphasizedPhrase2()
    {
        $inputString = "this phrase ".$this->formatChar.$this->formatChar."is".$this->formatChar.$this->formatChar." ".$this->formatChar.$this->formatChar."mismatched in an interesting way";

        $outputString = "<document><![CDATA[this phrase ]]>".$this->startTag."<![CDATA[is]]>".$this->endTag."<![CDATA[ ]]><![CDATA[".$this->formatChar.$this->formatChar."]]><![CDATA[mismatched in an interesting way]]></document>";

        
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //------------------------------------------------------------------
    //  Test "this phrase ***is*** ****mismatched in an interesting way"
    //------------------------------------------------------------------
    function testMismatchedEmphasizedPhrase3()
    {
        $inputString = "this phrase ".$this->formatChar.$this->formatChar.$this->formatChar."is".$this->formatChar.$this->formatChar.$this->formatChar." ".$this->formatChar.$this->formatChar.$this->formatChar.$this->formatChar."mismatched in an interesting way";

        $outputString = "<document><![CDATA[this phrase ]]><![CDATA[".$this->formatChar.$this->formatChar.$this->formatChar."]]><![CDATA[is]]><![CDATA[".$this->formatChar.$this->formatChar.$this->formatChar."]]><![CDATA[ ]]><![CDATA[".$this->formatChar.$this->formatChar.$this->formatChar.$this->formatChar."]]><![CDATA[mismatched in an interesting way]]></document>";

        
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------------
    //  Test "this phrase ****is**** ****mismatched in an interesting way"
    //--------------------------------------------------------------------
    function testMismatchedEmphasizedPhrase4()
    {
        $inputString = "this phrase ".$this->formatChar.$this->formatChar.$this->formatChar.$this->formatChar."is".$this->formatChar.$this->formatChar.$this->formatChar.$this->formatChar." ".$this->formatChar.$this->formatChar.$this->formatChar.$this->formatChar."mismatched in an interesting way";

        $outputString = "<document><![CDATA[this phrase ]]><![CDATA[".$this->formatChar.$this->formatChar.$this->formatChar.$this->formatChar."]]><![CDATA[is]]><![CDATA[".$this->formatChar.$this->formatChar.$this->formatChar.$this->formatChar."]]><![CDATA[ ]]><![CDATA[".$this->formatChar.$this->formatChar.$this->formatChar.$this->formatChar."]]><![CDATA[mismatched in an interesting way]]></document>";

        
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //------------------------------------------------------------------
    //  Test "A **Emphasize phrase\ncannot continue** across a line."
    //------------------------------------------------------------------
    function testEmphasizeEndsAtEoln()
    {
        $inputString = "A ".$this->formatChar.$this->formatChar."Emphasize phrase\ncannot continue".$this->formatChar.$this->formatChar." across a line.";
        $outputString = "<document><![CDATA[A ]]><![CDATA[".$this->formatChar.$this->formatChar."]]><![CDATA[Emphasize phrase\n]]><![CDATA[cannot continue]]><![CDATA[".$this->formatChar.$this->formatChar."]]><![CDATA[ across a line.]]></document>";

        
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
}

//--------------------------------------------------------------------------
//      Asterisk: Bold emphasis for text within paragraphs
//--------------------------------------------------------------------------
class ParserBoldAsteriskFormattingTest extends ParserEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserEmphasisFormattingTest :: __construct($name, "Bold", "*", "<b><origin><![CDATA[wiki]]></origin>", "</b>");
    }
}

//--------------------------------------------------------------------------
//      Slash: Italic emphasis for text within paragraphs
//--------------------------------------------------------------------------
class ParserItalicSlashFormattingTest extends ParserEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserEmphasisFormattingTest :: __construct($name, "Italics", "/", "<i><origin><![CDATA[wiki]]></origin>", "</i>");
    }
}

//--------------------------------------------------------------------------
//      Underline: Underscore emphasis for text within paragraphs
//--------------------------------------------------------------------------
class ParserUnderlineUnderbarFormattingTest extends ParserEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserEmphasisFormattingTest :: __construct($name, "Underline", "_", "<u><origin><![CDATA[wiki]]></origin>", "</u>");
    }
}

//--------------------------------------------------------------------------
//      Equals: Fixed font emphasis for text within paragraphs
//--------------------------------------------------------------------------
class ParserFixedwidthEqualsFormattingTest extends ParserEmphasisFormattingTest {
    function __construct($name)
    {
        ParserEmphasisFormattingTest :: __construct($name, "Fixed", "=", "<tt><origin><![CDATA[wiki]]></origin>", "</tt>");
    }
}

//--------------------------------------------------------------------------
//      Hyphen: Strikeout emphasis for text within paragraphs
//--------------------------------------------------------------------------
class ParserstrikeoutHyphenFormattingTest extends ParserEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserEmphasisFormattingTest :: __construct($name, "Fixed", "-", "<strike><origin><![CDATA[wiki]]></origin>", "</strike>");
    }
}

//--------------------------------------------------------------------------
//      Basic tests for formatting text with <tags> within paragraphs
//--------------------------------------------------------------------------
class ParserTagEmphasisFormattingTest extends PHPUnit_TestCase
{
    var $p = NULL;
    var $r = NULL;

    var $testType = "Undefined";
    var $startFormat = "<?>";
    var $endFormat = "</?>";
    var $startTag = "<start>";
    var $endTag = "</start>";

    //--------------------------------------------------------------
    //  Standard constructor for unit tests.
    //--------------------------------------------------------------
    function __construct($name, $type, $formatStart, $formatEnd, $start, $end)
    {
        $this->startFormat = $formatStart;
        $this->endFormat = $formatEnd;
        $this->startTag = $start;
        $this->endTag = $end;
        $this->PHPUnit_TestCase($name);
    }

    //--------------------------------------------------------------
    //  Setup: Set up an instance of the parser for testing
    //--------------------------------------------------------------
    function setUp()
    {
        $this->p = TestWikiParser :: getInstance();
        $this->r = TestReverseParser :: getInstance();
    }

    //--------------------------------------------------------------
    //  Helper function to generate input and output strings 
    //--------------------------------------------------------------
    function generateWikiEmphasis($emphasisStart, $emphasisEnd, $startTag, $endTag, $begin, $middle, $ending, & $input, & $output)
    {
        $input = $begin.$emphasisStart.$middle.$emphasisEnd.$ending;
        $output = "<document>".($begin!=''?"<![CDATA[".$begin."]]>":"").$startTag."<![CDATA[".$middle."]]>".$endTag.($ending!=''?"<![CDATA[".$ending."]]>":"")."</document>";
    }
    //--------------------------------------------------------------
    //  TODO:   Need to check how special characters like punctuation
    //                  affect emphasis.  "This *word!* should /emphasize/.
    //--------------------------------------------------------------

    //--------------------------------------------------------------
    //  Test emphasis on a single word
    //--------------------------------------------------------------
    function testEmphasizeWord()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiEmphasis($this->startFormat, $this->endFormat, $this->startTag, $this->endTag, "start ", "middle", " end", $inputString, $outputString);

        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test emphasis on a phrase
    //--------------------------------------------------------------
    function testEmphasizePhrase()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiEmphasis($this->startFormat, $this->endFormat, $this->startTag, $this->endTag, "start ", "middle words", " end", $inputString, $outputString);
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test emphasize an entire line (no terminator)
    //--------------------------------------------------------------
    function testEmphasizeLine()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiEmphasis($this->startFormat, $this->endFormat, $this->startTag, $this->endTag, "", "middle words", "", $inputString, $outputString);
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test mismatched emphasis
    //--------------------------------------------------------------
    function testMismatchedEmphasizedPhrase()
    {
        $inputString = "this phrase ".$this->startFormat."is mismatched";
        $outputString = "<document><![CDATA[this phrase ]]><![CDATA[".$this->startFormat."]]><![CDATA[is mismatched]]></document>";

        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test "this phrase <b>is</b> <b>mismatched in an interesting way"
    //--------------------------------------------------------------
    function testMismatchedEmphasizedPhrase2()
    {
        $inputString = "this phrase ".$this->startFormat."is".$this->endFormat." ".$this->startFormat."mismatched in an interesting way";

        $outputString = "<document><![CDATA[this phrase ]]>".$this->startTag."<![CDATA[is]]>".$this->endTag."<![CDATA[ ]]><![CDATA[".$this->startFormat."]]><![CDATA[mismatched in an interesting way]]></document>";

        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test "this phrase <b>is <b>mismatched</b> in an interesting way"
    //--------------------------------------------------------------
    function testMismatchedEmphasizedPhrase3()
    {
        $inputString = "this phrase ".$this->startFormat."is ".$this->startFormat."mismatched".$this->endFormat." in an interesting way";

        $outputString = "<document><![CDATA[this phrase ]]>".$this->startTag."<![CDATA[is ]]><![CDATA[".$this->startFormat."]]><![CDATA[mismatched]]>".$this->endTag."<![CDATA[ in an interesting way]]></document>";

        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test "this phrase <b>is <b>mismatched</b> in</b> an interesting way"
    //--------------------------------------------------------------
    function testMismatchedEmphasizedPhrase4()
    {
        $inputString = "this phrase ".$this->startFormat."is ".$this->startFormat."mismatched".$this->endFormat." in".$this->endFormat." an interesting way";

        $outputString = "<document><![CDATA[this phrase ]]>".$this->startTag."<![CDATA[is ]]><![CDATA[".$this->startFormat."]]><![CDATA[mismatched]]><![CDATA[".$this->endFormat."]]><![CDATA[ in]]>".$this->endTag."<![CDATA[ an interesting way]]></document>";

        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //------------------------------------------------------------------
    //  Test "A <b>Emphasize phrase\ncannot continue</b> across a line."
    //------------------------------------------------------------------
    function testEmphasizeEndsAtEoln()
    {
        $inputString = "A ".$this->startFormat."Emphasize phrase\ncannot continue".$this->endFormat." across a line.";
        $outputString = "<document><![CDATA[A ]]><![CDATA[".$this->startFormat."]]><![CDATA[Emphasize phrase\n]]><![CDATA[cannot continue]]><![CDATA[".$this->endFormat."]]><![CDATA[ across a line.]]></document>";

        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
}

//--------------------------------------------------------------------------
//      <b>: Bold emphasis for text within paragraphs
//--------------------------------------------------------------------------
class ParserBoldTagFormattingTest extends ParserTagEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserTagEmphasisFormattingTest :: __construct($name, "Bold", "<b>", "</b>", "<b><origin><![CDATA[html]]></origin>", "</b>");
    }
}

//--------------------------------------------------------------------------
//      <i>: Italic emphasis for text within paragraphs
//--------------------------------------------------------------------------
class ParserItalicTagFormattingTest extends ParserTagEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserTagEmphasisFormattingTest :: __construct($name, "Italics", "<i>", "</i>", "<i><origin><![CDATA[html]]></origin>", "</i>");
    }
}

//--------------------------------------------------------------------------
//      <u>: Underscore emphasis for text within paragraphs
//--------------------------------------------------------------------------
class ParserUnderlineTagFormattingTest extends ParserTagEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserTagEmphasisFormattingTest :: __construct($name, "Underline", "<u>", "</u>", "<u><origin><![CDATA[html]]></origin>", "</u>");
    }
}

//--------------------------------------------------------------------------
//      <tt>: Fixed font emphasis for text within paragraphs
//--------------------------------------------------------------------------
class ParserFixedwidthTagFormattingTest extends ParserTagEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserTagEmphasisFormattingTest :: __construct($name, "Fixed", "<tt>", "</tt>", "<tt><origin><![CDATA[html]]></origin>", "</tt>");
    }
}

//--------------------------------------------------------------------------
//      <s>: Strikeout emphasis for text within paragraphs
//--------------------------------------------------------------------------
class ParserStrikeoutTagFormattingTest extends ParserTagEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserTagEmphasisFormattingTest :: __construct($name, "Strikeout", "<strike>", "</strike>", "<strike><origin><![CDATA[html]]></origin>", "</strike>");
    }
}

//--------------------------------------------------------------------------
//      <sub>: Subscript for text within paragraphs
//--------------------------------------------------------------------------
class ParserSubscriptTagFormattingTest extends ParserTagEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserTagEmphasisFormattingTest :: __construct($name, "Subscript", "<sub>", "</sub>", "<sub><origin><![CDATA[html]]></origin>", "</sub>");
    }
}

//--------------------------------------------------------------------------
//      <sup>: Superscript for text within paragraphs
//--------------------------------------------------------------------------
class ParserSuperscriptTagFormattingTest extends ParserTagEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserTagEmphasisFormattingTest :: __construct($name, "Superscript", "<sup>", "</sup>", "<sup><origin><![CDATA[html]]></origin>", "</sup>");
    }
}

//--------------------------------------------------------------------------
//      <left>: Left justification for text within paragraphs
//--------------------------------------------------------------------------
class ParserLeftJustifyTagFormattingTest extends ParserTagEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserTagEmphasisFormattingTest :: __construct($name, "Left justify", "<left>", "</left>", "<left><origin><![CDATA[html]]></origin>", "</left>");
    }
}

//--------------------------------------------------------------------------
//      <center>: Center justification for text within paragraphs
//--------------------------------------------------------------------------
class ParserCenterJustifyTagFormattingTest extends ParserTagEmphasisFormattingTest
{
    function __construct($name)
    {
                ParserTagEmphasisFormattingTest :: __construct($name, "Center justify", "<center>", "</center>", "<center><origin><![CDATA[html]]></origin>", "</center>");
    }
}

//--------------------------------------------------------------------------
//      <right>: Right justification for text within paragraphs
//--------------------------------------------------------------------------
class ParserRightJustifyTagFormattingTest extends ParserTagEmphasisFormattingTest
{
    function __construct($name)
    {
        ParserTagEmphasisFormattingTest :: __construct($name, "Right justify", "<right>", "</right>", "<right><origin><![CDATA[html]]></origin>", "</right>");
    }
}

//--------------------------------------------------------------------------
//      <bad>: Should handle bad tags OK!
//--------------------------------------------------------------------------
class ParserBadTagFormattingTest extends PHPUnit_TestCase {
    var $p = NULL;
    var $r = NULL;

    var $testType = "Bad Tag";
    var $startFormat = "<bad>";
    var $endFormat = "</bad>";
    var $startTag = "";
    var $endTag = "";

    //--------------------------------------------------------------
    //  Standard constructor for unit tests.
    //--------------------------------------------------------------
    function __construct($name)
    {
        $this->startTag = htmlentities($this->startFormat);
        $this->endTag = htmlentities($this->endFormat);
        $this->PHPUnit_TestCase($name);
    }

    //--------------------------------------------------------------
    //  Setup: Set up an instance of the parser for testing
    //--------------------------------------------------------------
    function setUp()
    {
        $this->p = TestWikiParser :: getInstance();
        $this->r = TestReverseParser :: getInstance();
    }

    //--------------------------------------------------------------
    //  Test "this phrase <bad>is</bad> <bad>mismatched in an interesting way"
    //--------------------------------------------------------------
    function testMismatchedEmphasizedPhrase2()
    {
        $inputString = "this phrase ".$this->startFormat."is".$this->endFormat." ".$this->startFormat."mismatched in an interesting way";
        $outputString = "<document><![CDATA[this phrase ]]><![CDATA[".$this->startTag."]]><![CDATA[is]]><![CDATA[".$this->endTag."]]><![CDATA[ ]]><![CDATA[".$this->startTag."]]><![CDATA[mismatched in an interesting way]]></document>";

        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
}


//--------------------------------------------------------------------------
//      Basic tests for formatting text within <tags> within paragraphs
//--------------------------------------------------------------------------
class ParserGreedyTagFormattingTest extends PHPUnit_TestCase {
        var $p = NULL;
        var $r = NULL;

        var $testType = "Undefined";
        var $startFormat = "<?>";
        var $endFormat = "</?>";
        var $startTag = "<start>";
        var $endTag = "</start>";

    //--------------------------------------------------------------
    //  Standard constructor for unit tests.
    //--------------------------------------------------------------
    function __construct($name, $type, $formatStart, $formatEnd, $start, $end)
    {
        $this->startFormat = $formatStart;
        $this->endFormat = $formatEnd;
        $this->startTag = $start;
        $this->endTag = $end;
        $this->PHPUnit_TestCase($name);
    }

    //--------------------------------------------------------------
    //  Setup: Set up an instance of the parser for testing
    //--------------------------------------------------------------
    function setUp()
    {
        $this->p = TestWikiParser :: getInstance();
        $this->r = TestReverseParser :: getInstance();
    }

    //--------------------------------------------------------------
    //  Helper function to generate input and output strings 
    //--------------------------------------------------------------
    function generateWikiGreedy($greedyStart, $greedyEnd, $startTag, $endTag, $begin, $middle, $ending, & $input, & $output)
    {
        $input = $begin.$greedyStart.$middle.$greedyEnd.$ending;
        $output = "<document>".($begin!=''?"<![CDATA[".$begin."]]>":"").$startTag."<![CDATA[".$middle."]]>".$endTag.($ending!=''?"<![CDATA[".$ending."]]>":"")."</document>";
    }

    //--------------------------------------------------------------
    //  Test greediness on a single word
    //--------------------------------------------------------------
    function testGreedyWord()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiGreedy($this->startFormat, $this->endFormat, $this->startTag, $this->endTag, "start ", "middle", " end", $inputString, $outputString);
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //--------------------------------------------------------------
    //  Test greediness on a phrase
    //--------------------------------------------------------------
    function testGreedyPhrase()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiGreedy($this->startFormat, $this->endFormat, $this->startTag, $this->endTag, "start ", "middle words", " end", $inputString, $outputString);
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //--------------------------------------------------------------
    //  Test emphasize an entire line (no terminator)
    //--------------------------------------------------------------
    function testGreedyLine()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiGreedy($this->startFormat, $this->endFormat, $this->startTag, $this->endTag, "", "middle words", "", $inputString, $outputString);
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //--------------------------------------------------------------
    //  Test mismatched greedy tags
    //--------------------------------------------------------------
    function testMismatchedGreedyPhrase()
    {
        $inputString = "this phrase ".$this->startFormat."is mismatched";
        $outputString = "<document><![CDATA[this phrase ]]>".$this->startTag."<![CDATA[is mismatched]]>Mismatched '".$this->startFormat."': missing end tag.</document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
    }
    //--------------------------------------------------------------
    //  Test "this phrase <tag>is</tag> <tag>mismatched in an interesting way"
    //--------------------------------------------------------------
    function testMismatchedGreedyPhrase2()
    {
        $inputString = "this phrase ".$this->startFormat."is".$this->endFormat." ".$this->startFormat."mismatched in an interesting way";
        $outputString = "<document><![CDATA[this phrase ]]>".$this->startTag."<![CDATA[is]]>".$this->endTag."<![CDATA[ ]]>".$this->startTag."<![CDATA[mismatched in an interesting way]]>Mismatched '".$this->startFormat."': missing end tag.</document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
    }
    //------------------------------------------------------------------
    //  Test "A <tag>Greedy phrase \n can continue</tag> across a line."
    //------------------------------------------------------------------
    function testGreedyOverEoln()
    {
        $inputString = "A ".$this->startFormat."Greedy phrase \ncan continue".$this->endFormat." across a line.";
        $outputString = "<document><![CDATA[A ]]>".$this->startTag."<![CDATA[Greedy phrase \n]]><![CDATA[can continue]]>".$this->endTag."<![CDATA[ across a line.]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
}

//--------------------------------------------------------------------------
//      <noop>: No-operation tag for text within a paragraph
//--------------------------------------------------------------------------
class ParserNoopTagFormattingTest extends ParserGreedyTagFormattingTest
{
    function __construct($name)
    {
        ParserGreedyTagFormattingTest :: __construct($name, "Noop", "<noop>", "</noop>", '<noop>', "</noop>");
    }
}
//--------------------------------------------------------------------------
//      <rem>: Remark tag for text within a paragraph
//--------------------------------------------------------------------------
class ParserRemTagFormattingTest extends ParserGreedyTagFormattingTest {
    function __construct($name)
    {
        ParserGreedyTagFormattingTest :: __construct($name, "Rem", "<rem>", "</rem>", '<rem>', "</rem>");
    }
}
//--------------------------------------------------------------------------
//      <code>: Code-formatting tag for text within a paragraph
//--------------------------------------------------------------------------
class ParserCodeTagFormattingTest extends ParserGreedyTagFormattingTest
{
    function __construct($name)
    {
        ParserGreedyTagFormattingTest :: __construct($name, "Code", "<code>", "</code>", '<code>', "</code>");
    }
}
//--------------------------------------------------------------------------
//      <pre>: Pre-formatted tag for text within a paragraph
//--------------------------------------------------------------------------
class ParserPreTagFormattingTest extends ParserGreedyTagFormattingTest
{
    function __construct($name)
    {
      ParserGreedyTagFormattingTest :: __construct($name, "Pre", "<pre>", "</pre>", '<pre>', "</pre>");
    }
}

//--------------------------------------------------------------------------
//      Basic tests for formatting text within <tags> within paragraphs
//--------------------------------------------------------------------------
class ParserGreedyContentTagFormattingTest extends PHPUnit_TestCase {
    var $p = NULL;
    var $r = NULL;

    var $testType = "Undefined";
    var $startFormat = "<?>";
    var $endFormat = "</?>";
    var $startTag = "<start>";
    var $endTag = "</start>";
    var $startEmph = "<?>";
    var $endEmph = "</?>";
    var $startEmTag = "<start>";
    var $endEmTag = "</start>";

    //--------------------------------------------------------------
    //  Standard constructor for unit tests.
    //--------------------------------------------------------------
    function __construct($name, $type, $formatStart, $formatEnd, $start, $end,
                            $emphStart, $emphEnd, $startEm, $endEm )
    {
        $this->startFormat = $formatStart;
        $this->endFormat = $formatEnd;
        $this->startTag = $start;
        $this->endTag = $end;
        $this->startEmph = $emphStart;
        $this->endEmph = $emphEnd;
        $this->startEmTag = $startEm;
        $this->endEmTag = $endEm;
        $this->PHPUnit_TestCase($name);
    }

    //--------------------------------------------------------------
    //  Setup: Set up an instance of the parser for testing
    //--------------------------------------------------------------
    function setUp()
    {
        $this->p = TestWikiParser :: getInstance();
        $this->r = TestReverseParser :: getInstance();
    }

    //--------------------------------------------------------------
    //  Helper function to generate input and output strings 
    //--------------------------------------------------------------
    function generateWikiGreedy($greedyStart, $greedyEnd, $startTag, $endTag, $begin, $middle, $ending, & $input, & $output)
    {
        $input = $begin.$greedyStart.$middle.$greedyEnd.$ending;
        $output = "<document>".($begin!=''?"<![CDATA[".$begin."]]>":"").$startTag."<![CDATA[".$middle."]]>".$endTag.($ending!=''?"<![CDATA[".$ending."]]>":"")."</document>";
    }

    //--------------------------------------------------------------
    //  Test greediness on a single word
    //--------------------------------------------------------------
    function testGreedyWord()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiGreedy($this->startFormat, $this->endFormat, $this->startTag, $this->endTag, "start ", "middle", " end", $inputString, $outputString);
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test greediness on a phrase
    //--------------------------------------------------------------
    function testGreedyPhrase()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiGreedy($this->startFormat, $this->endFormat, $this->startTag, $this->endTag, "start ", "middle words", " end", $inputString, $outputString);
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test emphasize an entire line (no terminator)
    //--------------------------------------------------------------
    function testGreedyLine()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiGreedy($this->startFormat, $this->endFormat, $this->startTag, $this->endTag, "", "middle words", "", $inputString, $outputString);
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //--------------------------------------------------------------
    //  Test "this phrase <tag>is mismatched"
    //--------------------------------------------------------------
    function testMismatchedGreedyPhrase()
    {
        $inputString = "this phrase ".$this->startFormat."is mismatched";
        $outputString = "<document><![CDATA[this phrase ]]>".$this->startFormat."<![CDATA[is mismatched]]>Mismatched '".$this->startFormat."': missing end tag.</document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
    }

    //--------------------------------------------------------------
    //  Test "this phrase <tag>is</tag> <tag>mismatched in an interesting way"
    //--------------------------------------------------------------
    function testMismatchedGreedyPhrase2()
    {
        $inputString = "this phrase ".$this->startFormat."is".$this->endFormat." ".$this->startFormat."mismatched in an interesting way";
        $outputString = "<document><![CDATA[this phrase ]]>".$this->startTag."<![CDATA[is]]>".$this->endTag."<![CDATA[ ]]>".$this->startFormat."<![CDATA[mismatched in an interesting way]]>Mismatched '".$this->startFormat."': missing end tag.</document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
    }

    //------------------------------------------------------------------
    //  Test "A <tag>Greedy phrase \n can continue</tag> across a line."
    //------------------------------------------------------------------
    function testGreedyOverEoln()
    {
        $inputString = "A ".$this->startFormat."Greedy phrase \ncan continue".$this->endFormat." across a line.";
        $outputString = "<document><![CDATA[A ]]>".$this->startTag."<![CDATA[Greedy phrase \n]]><![CDATA[can continue]]>".$this->endTag."<![CDATA[ across a line.]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //--------------------------------------------------------------
    //  Helper function to generate input and output strings 
    //--------------------------------------------------------------
    function generateWikiGreedyEmphasis($greedyStart, $greedyEnd, $emphasisStart, $emphasisEnd, $startTag, $endTag, $startEmph, $endEmph, $begin, $middle1, $middle2, $middle3, $ending, & $input, & $output)
    {
        $input = $begin.$greedyStart.$middle1.$emphasisStart.$middle2.$emphasisEnd.$middle3.$greedyEnd.$ending;
        $output = "<document>".($begin!=''?"<![CDATA[".$begin."]]>":"").$startTag.($middle1!=''?"<![CDATA[".$middle1."]]>":"").$startEmph.($middle2!=''?"<![CDATA[".$middle2."]]>":"").$endEmph.($middle3!=''?"<![CDATA[".$middle3."]]>":"").$endTag.($ending!=''?"<![CDATA[".$ending."]]>":"")."</document>";
    }
    //--------------------------------------------------------------
    //  TODO:   Need to check how special characters like punctuation
    //                  affect emphasis.  "This *word!* should /emphasize/.
    //--------------------------------------------------------------

    //--------------------------------------------------------------
    //  Test emphasis on a single word
    //--------------------------------------------------------------
    function testEmphasizeWord()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiGreedyEmphasis($this->startFormat, $this->endFormat, $this->startEmph, $this->endEmph, $this->startTag, $this->endTag, $this->startEmTag, $this->endEmTag, "start ", "middle1", "middle2", "middle3", " end", $inputString, $outputString);
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test emphasis on a phrase
    //--------------------------------------------------------------
    function testEmphasizePhrase()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiGreedyEmphasis($this->startFormat, $this->endFormat, $this->startEmph, $this->endEmph, $this->startTag, $this->endTag, $this->startEmTag, $this->endEmTag, "start ", "middle1", "middle2 words", "middle3", " end", $inputString, $outputString);
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test emphasize an entire line (no terminator)
    //--------------------------------------------------------------
    function testEmphasizeLine()
    {
        $inputString = "";
        $outputString = "";
        $this->generateWikiGreedyEmphasis($this->startFormat, $this->endFormat, $this->startEmph, $this->endEmph, $this->startTag, $this->endTag, $this->startEmTag, $this->endEmTag, "", "", "middle2 words", "", "", $inputString, $outputString);
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //--------------------------------------------------------------
    //  Test mismatched emphasis
    //--------------------------------------------------------------
    function testMismatchedEmphasizedPhrase()
    {
        $inputString = $this->startFormat."this phrase ".$this->startEmph."is mismatched".$this->endFormat;
        $outputString = "<document>".$this->startTag."<![CDATA[this phrase ]]><![CDATA[".$this->startEmph."]]><![CDATA[is mismatched]]>".$this->endTag."</document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //--------------------------------------------------------------
    //  Test "this phrase <b>is</b> <b>mismatched in an interesting way"
    //--------------------------------------------------------------
    function testMismatchedEmphasizedPhrase2()
    {
        $inputString = $this->startFormat."this phrase ".$this->startEmph."is".$this->endEmph." ".$this->startEmph."mismatched in an interesting way".$this->endFormat;
        $outputString = "<document>".$this->startTag."<![CDATA[this phrase ]]>".$this->startEmTag."<![CDATA[is]]>".$this->endEmTag."<![CDATA[ ]]><![CDATA[".$this->startEmph."]]><![CDATA[mismatched in an interesting way]]>".$this->endFormat."</document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //------------------------------------------------------------------
    //  Test "A **Emphasize phrase\ncannot continue** across a line."
    //------------------------------------------------------------------
    function testEmphasizeEndsAtEoln()
    {
        $inputString = $this->startFormat."A ".$this->startEmph."Emphasize phrase\ncannot continue".$this->endEmph." across a line.".$this->endFormat;
        $outputString = "<document>".$this->startTag."<![CDATA[A ]]><![CDATA[".$this->startEmph."]]><![CDATA[Emphasize phrase\n]]><![CDATA[cannot continue]]><![CDATA[".$this->endEmph."]]><![CDATA[ across a line.]]>".$this->endTag."</document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
}


//--------------------------------------------------------------------------
//      <posting>: Posting-formatted tag for text within a paragraph
//--------------------------------------------------------------------------
class ParserPostingTagFormattingTest extends ParserGreedyContentTagFormattingTest {
    function __construct($name)
    {
        ParserGreedyContentTagFormattingTest :: __construct($name, "Posting", "<posting>", "</posting>", '<posting>', "</posting>", "<b>", "</b>", "<b><origin><![CDATA[html]]></origin>", '</b>' );
    }
}
//--------------------------------------------------------------------------
//      <q>: Quote tag for text within a paragraph
//--------------------------------------------------------------------------
class ParserQTagFormattingTest extends ParserGreedyContentTagFormattingTest {
    function __construct($name)
    {
        ParserGreedyContentTagFormattingTest :: __construct($name, "Q", "<q>", "</q>", '<q>', "</q>", "<b>", "</b>", "<b><origin><![CDATA[html]]></origin>", '</b>' );
    }
}

//--------------------------------------------------------------------------
//      <br/>, variables, links, uris & plugins
//--------------------------------------------------------------------------
class ParserOtherProcessContentFormattingTest extends PHPUnit_TestCase {
    var $p = NULL;
    var $r = NULL;
    //--------------------------------------------------------------
    //  Standard constructor for unit tests.
    //--------------------------------------------------------------
    function __construct($name)
    {
        $this->PHPUnit_TestCase($name);
    }

    //--------------------------------------------------------------
    //  Setup: Set up an instance of the parser for testing
    //--------------------------------------------------------------
    function setUp()
    {
        $this->p = TestWikiParser :: getInstance();
        $this->r = TestReverseParser :: getInstance();
    }

    //------------------------------------------------------------------
    //  <br/> (needn't include slash, will be added)
    //------------------------------------------------------------------
    function testLineBreak()
    {
        $inputString = "A line<br    >-break";
        $outputString = "<document><![CDATA[A line]]><br/><![CDATA[-break]]></document>";
        $reverseString = "A line<br/>-break";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($reverseString, $this->r->parse($outputString));
    }

    //------------------------------------------------------------------
    //  variables
    //------------------------------------------------------------------
    function testVariable()
    {
        $inputString = 'A %VARIABLE% test';
        $outputString = "<document><![CDATA[A ]]><var><name><![CDATA[VARIABLE]]></name></var><![CDATA[ test]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testVariableSolo()
    {
        $inputString = '%VARIABLE%';
        $outputString = "<document><var><name><![CDATA[VARIABLE]]></name></var></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals(trim($inputString), trim($this->r->parse($outputString)));
    }
    function testPercent()
    {
        $inputString = 'A 100% test';
        $outputString = "<document><![CDATA[A 100% test]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testVariableFaulty()
    {
        $inputString = "A %LINE\nBREAK% test";
        $outputString = "<document><![CDATA[A %LINE\n]]><![CDATA[BREAK% test]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //------------------------------------------------------------------
    //  links
    //------------------------------------------------------------------
    function testBrokenLinkSyntax1()
    {
        $inputString = "A ((http://www.google.co.uk)\n(Alias)) test";
        $outputString = "<document><![CDATA[A ((]]><uri><strref><![CDATA[http://www.google.co.uk]]></strref></uri><![CDATA[)\n]]><![CDATA[(Alias)) test]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testBrokenLinkSyntax2()
    {
        $inputString = "A ((http://www.goo\ngle.co.uk)(Alias)) test";
        $outputString = "<document><![CDATA[A ((]]><uri><strref><![CDATA[http://www.goo]]></strref></uri><![CDATA[\n]]><![CDATA[gle.co.uk)(Alias)) test]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testBrokenLinkSyntax3()
    {
        $inputString = "A ((http://www.google.co.uk)(Al\nias)) test";
        $outputString = "<document><![CDATA[A ((]]><uri><strref><![CDATA[http://www.google.co.uk]]></strref></uri><![CDATA[)(Al\n]]><![CDATA[ias)) test]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals(trim($inputString), trim($this->r->parse($outputString)));
    }
    function testUriLinkElement()
    {
        $inputString = 'A ((http://www.google.co.uk)(Alias)) test';
        $outputString = "<document><![CDATA[A ]]><link><origin><![CDATA[round]]></origin><href><![CDATA[http://www.google.co.uk]]></href><![CDATA[Alias]]></link><![CDATA[ test]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals(trim($inputString), trim($this->r->parse($outputString)));
    }
    function testGlobalLinkElement()
    {
        $inputString = 'A ((Web|Ref)(Alias)) test';
        $outputString = "<document><![CDATA[A ]]><link><origin><![CDATA[round]]></origin><strref><![CDATA[WebRef]]></strref><![CDATA[Alias]]></link><![CDATA[ test]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testLocalLinkElement()
    {
        $inputString = 'A [[Ref][Alias]] test';
        $outputString = "<document><![CDATA[A ]]><link><origin><![CDATA[square]]></origin><strref><![CDATA[Ref]]></strref><![CDATA[Alias]]></link><![CDATA[ test]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //------------------------------------------------------------------
    //  CamelCase
    //------------------------------------------------------------------
    function testCamelCase()
    {
        $inputString = 'A CamelCase link';
        $outputString = "<document><![CDATA[A ]]><link><origin><![CDATA[camel]]></origin><strref><![CDATA[Camel Case]]></strref><![CDATA[CamelCase]]></link><![CDATA[ link]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //------------------------------------------------------------------
    //  URIs
    //------------------------------------------------------------------
    function testURIElement()
    {
        $inputString = 'some text http://cowiki.tigris.org';
        $outputString = "<document><![CDATA[some text ]]><uri><strref><![CDATA[http://cowiki.tigris.org]]></strref></uri></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testURIElementAlone()
    {
        $inputString = 'http://cowiki.tigris.org';
        $outputString = "<document><uri><strref><![CDATA[http://cowiki.tigris.org]]></strref></uri></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testURIBrokenElement()
    {
        $inputString = "text http://co\nwiki.tigris.org";
        $outputString = "<document><![CDATA[text ]]><uri><strref><![CDATA[http://co]]></strref></uri><![CDATA[\n]]><![CDATA[wiki.tigris.org]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testURIBrokenElement2()
    {
        $inputString = "text ht\ntp://cowiki.tigris.org";
        $outputString = "<document><![CDATA[text ht\n]]><uri><strref><![CDATA[tp://cowiki.tigris.org]]></strref></uri></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //------------------------------------------------------------------
    //  plugins
    //------------------------------------------------------------------
    function testPluginElement()
    {
        $inputString = 'text <plugin Example>';
        $outputString = "<document><![CDATA[text ]]><plugin><name><![CDATA[Example]]></name></plugin></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testPluginElementSlash() //slash will be removed
    {
        $inputString = 'text <plugin Example/>';
        $outputString = "<document><![CDATA[text ]]><plugin><name><![CDATA[Example]]></name></plugin></document>";
        $revString = 'text <plugin Example>';
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($revString, $this->r->parse($outputString));
    }
    function testPluginElementBroken()
    {
        $inputString = "text <plu\n \ngin Example/>";
        $outputString = "<document><![CDATA[text &lt;plu\n]]><![CDATA[ \n]]><![CDATA[gin Example/&gt;]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals(trim($inputString), trim($this->r->parse($outputString)));
    }
    function testPluginElementBroken2()
    {
        $inputString = "text <plugin Exa\nmple/>";
        $outputString = "<document><![CDATA[text &lt;plugin Exa\n]]><![CDATA[mple/&gt;]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testPluginElementBroken3()
    {
        $inputString = "text <plugin\nExample/>";
        $outputString = "<document><![CDATA[text &lt;plugin\n]]><![CDATA[Example/&gt;]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testPluginElementAttributed()
    {
        $inputString = "text <plugin Example attr=\"val\">";
        $outputString = "<document><![CDATA[text ]]><plugin><name><![CDATA[Example]]></name><attr><![CDATA[val]]></attr></plugin></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testPluginElementMoreAttributed()
    {
        $inputString = "text <plugin Example attr=\"val\" attr2=\"val2\">";
        $outputString = "<document><![CDATA[text ]]><plugin><name><![CDATA[Example]]></name><attr><![CDATA[val]]></attr><attr2><![CDATA[val2]]></attr2></plugin></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
}

//--------------------------------------------------------------------------
//      Headings, tables, lists
//--------------------------------------------------------------------------
class ParserOtherParseFormattingTest extends PHPUnit_TestCase {
    var $p = NULL;
    var $r = NULL;

    //--------------------------------------------------------------
    //  Standard constructor for unit tests.
    //--------------------------------------------------------------
    function __construct($name)
    {
        $this->PHPUnit_TestCase($name);
    }

    //--------------------------------------------------------------
    //  Setup: Set up an instance of the parser for testing
    //--------------------------------------------------------------
    function setUp()
    {
        $this->p = TestWikiParser :: getInstance();
        $this->r = TestReverseParser :: getInstance();
    }
    //------------------------------------------------------------------
    //  Headings
    //------------------------------------------------------------------
    function testHeadingNoSpace()
    {
        $inputString = "+++Heading3";
        $outputString = "<document><h3><![CDATA[Heading3]]></h3></document>";
        $revString = "+++ Heading3";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($revString, $this->r->parse($outputString));
    }
    function testHeadingSpace()
    {
        $inputString = "+++ Heading3\nmore text";
        $outputString = "<document><h3><![CDATA[Heading3]]></h3><![CDATA[\n]]><![CDATA[more text]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testHeadingLeadSpace()
    {
        $inputString = " +++ Heading3\nmore text";
        $outputString = "<document><![CDATA[ +++ Heading3\n]]><![CDATA[more text]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testHeadingEmphasis()
    {
        $inputString = "+++ **Heading3 bold**\nmore text";
        $outputString = "<document><h3><b><origin><![CDATA[wiki]]></origin><![CDATA[Heading3 bold]]></b></h3><![CDATA[\n]]><![CDATA[more text]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testHeadingJustification()
    {
        $inputString = "+++ <right>Heading3</right>";
        $outputString = "<document><h3><right><origin><![CDATA[html]]></origin><![CDATA[Heading3]]></right></h3></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testHeadingWrongJustification()
    {
        $inputString = "<right>+++ Heading3</right>\nmore text";
        $outputString = "<document><right><origin><![CDATA[html]]></origin><![CDATA[+++ Heading3]]></right><![CDATA[\n]]><![CDATA[more text]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testHeadingLineBreak()
    {
        $inputString = "+++ Head<br/>ing\nmore text";
        $outputString = "<document><h3><![CDATA[Head]]><br/><![CDATA[ing]]></h3><![CDATA[\n]]><![CDATA[more text]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testHeadingVariable()
    {
        $inputString = "+++ %TITLE%\nmore text";
        $outputString = "<document><h3><var><name><![CDATA[TITLE]]></name></var></h3><![CDATA[\n]]><![CDATA[more text]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testHeadingVariable2()
    {
        $inputString = "+ %TITLE%\nmore text";
        $outputString = "<document><h1><var><name><![CDATA[TITLE]]></name></var></h1><![CDATA[\n]]><![CDATA[more text]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testHeadingLink()
    {
        $inputString = "+++ ((www.h2g2.com)(H2G2))\nmore text";
        $outputString = "<document><h3><link><origin><![CDATA[round]]></origin><strref><![CDATA[www.h2g2.com]]></strref><![CDATA[H2G2]]></link></h3><![CDATA[
]]><![CDATA[more text]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testHeadingURI()
    {
        $inputString = "+++ http://www.h2g2.com\nmore text";
        $outputString = "<document><h3><uri><strref><![CDATA[http://www.h2g2.com]]></strref></uri></h3><![CDATA[\n]]><![CDATA[more text]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testHeadingPlugin()
    {
        $inputString = "+++ <plugin Example>";
        $outputString = "<document><h3><plugin><name><![CDATA[Example]]></name></plugin></h3></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //------------------------------------------------------------------
    //  Tables
    //------------------------------------------------------------------
    function testTablesCaptionBroken()
    {
        $inputString = "test <table>\n|+Caption\n</table>";
        $outputString = "<document><![CDATA[test ]]><![CDATA[&lt;table&gt;]]><![CDATA[\n]]><![CDATA[|+Caption\n]]><![CDATA[&lt;/table&gt;]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesCaption()
    {
        $inputString = "<table>\n|+Caption\n</table>";
        $outputString = "<document><table><caption><![CDATA[Caption]]></caption></table></document>";
        $revString = "<table>\n|+ Caption\n</table>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($revString, $this->r->parse($outputString));
    }
    function testTablesCaptionDouble()
    {
        $inputString = "<table>\n|+ Caption\n|+ Caption2\n</table>";
        $outputString = "<document><table><caption><![CDATA[Caption]]></caption><caption><![CDATA[Caption2]]></caption></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesAttributed()
    {
        $inputString = "<table attr = \"val\">\n|+ Caption\n|+ Caption2\n</table>";
        $outputString = "<document><table><attr><![CDATA[val]]></attr><caption><![CDATA[Caption]]></caption><caption><![CDATA[Caption2]]></caption></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesRowParamBroken()
    {
        $inputString = "<table>\n|-rowparam = \"detail\"\n| cell\n</table>";
        $outputString = "<document><table><tr><rowparam><![CDATA[detail]]></rowparam><td><![CDATA[cell]]></td></tr></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesRowParamBroken2()
    {
        $inputString = "<table>\n|-rowparam = \"detail\" | cell\n</table>";
        $outputString = "<document><table><tr><rowparam><![CDATA[detail]]></rowparam></tr></table></document>";
        $revString = "<table>\n|-rowparam = \"detail\"\n</table>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($revString, $this->r->parse($outputString));
    }
    function testTablesRowParamCell()
    {
        $inputString = "<table>\n|-rowparam = \"detail\"\n| cell\n</table>";
        $outputString = "<document><table><tr><rowparam><![CDATA[detail]]></rowparam><td><![CDATA[cell]]></td></tr></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesRowCellParam()
    {
        $inputString = "<table>\n|-\n|cellparam = \"detail\"| cell\n</table>";
        $outputString = "<document><table><tr><td><cellparam><![CDATA[detail]]></cellparam><![CDATA[ cell]]></td></tr></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesRowHeadParam()
    {
        $inputString = "<table>\n|-\n!cellparam = \"detail\"|head\n</table>";
        $outputString = "<document><table><tr><th><cellparam><![CDATA[detail]]></cellparam><![CDATA[head]]></th></tr></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //------------------------------------------------------------------
    //  nospace, space, emphasis, justification, linebreak, link, uri, plugin
    //------------------------------------------------------------------
    function testTablesEmphasis()
    {
        $inputString = "<table>\n| some **bold** text\n</table>";
        $outputString = "<document><table><td><![CDATA[some ]]><b><origin><![CDATA[wiki]]></origin><![CDATA[bold]]></b><![CDATA[ text]]></td></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesJustification()
    {
        $inputString = "<table>\n| <right>head</right>\n</table>";
        $outputString = "<document><table><td><right><origin><![CDATA[html]]></origin><![CDATA[head]]></right></td></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesLineBreak()
    {
        $inputString = "<table>\n| some<br/>text\n</table>";
        $outputString = "<document><table><td><![CDATA[some]]><br/><![CDATA[text]]></td></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesLink()
    {
        $inputString = "<table>\n|| some ((Web|Ref)(Alias)) text\n</table>";
        $outputString = "<document><table><td><origin><![CDATA[doubled]]></origin><![CDATA[some ]]><link><origin><![CDATA[round]]></origin><strref><![CDATA[WebRef]]></strref><![CDATA[Alias]]></link><![CDATA[ text]]></td></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesLink2()
    {
        $inputString = "<table>\n|cellparam = \"doh\"| some ((Web|Ref)(Alias)) text\n</table>";
        $outputString = "<document><table><td><cellparam><![CDATA[doh]]></cellparam><![CDATA[ some ]]><link><origin><![CDATA[round]]></origin><strref><![CDATA[WebRef]]></strref><![CDATA[Alias]]></link><![CDATA[ text]]></td></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesURI()
    {
        $inputString = "<table>\n| text http://www.h2g2.com\n</table>";
        $outputString = "<document><table><td><![CDATA[text ]]><uri><strref><![CDATA[http://www.h2g2.com]]></strref></uri></td></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTablesPlugin()
    {
        $inputString = "<table>\n| text <plugin Example>\n</table>";
        $outputString = "<document><table><td><![CDATA[text ]]><plugin><name><![CDATA[Example]]></name></plugin></td></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //------------------------------------------------------------------
    //  Lists & Tables
    //------------------------------------------------------------------
    function testListInTable()
    {
        $inputString = "<table>\n|\n# First\n## Second\n</table>";
        $outputString = "<document><table><td><list><ol><li><![CDATA[First]]><ol><li><![CDATA[Second]]></li></ol></li></ol></list></td></table></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //------------------------------------------------------------------
    //  Lists
    //------------------------------------------------------------------
    function testList()
    {
        $inputString = "# List first\n# List second\n";
        $outputString = "<document><list><ol><li><![CDATA[List first]]></li><li><![CDATA[List second]]></li></ol></list></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testLists()
    {
        $inputString = "# List first\n## List second\nNormal <right>text</right>";
        $outputString = "<document><list><ol><li><![CDATA[List first]]><ol><li><![CDATA[List second]]></li></ol></li></ol></list><![CDATA[Normal ]]><right><origin><![CDATA[html]]></origin><![CDATA[text]]></right></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testListNoSpace()
    {
        $inputString = "#List first\n##List second\n";
        $outputString = "<document><![CDATA[#List first\n]]><![CDATA[##List second\n]]></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testListBullet()
    {
        $inputString = "* List first\n** List second\n";
        $outputString = "<document><list><ul><li><![CDATA[List first]]><ul><li><![CDATA[List second]]></li></ul></li></ul></list></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //------------------------------------------------------------------
    //  nospace, space, emphasis, justification, linebreak, link, uri, plugin
    //------------------------------------------------------------------
    function testListEmphasis()
    {
        $inputString = "# some **bold** text\n# more //emphasised// text\n";
        $outputString = "<document><list><ol><li><![CDATA[some ]]><b><origin><![CDATA[wiki]]></origin><![CDATA[bold]]></b><![CDATA[ text]]></li><li><![CDATA[more ]]><i><origin><![CDATA[wiki]]></origin><![CDATA[emphasised]]></i><![CDATA[ text]]></li></ol></list></document>";
        $this->assertEquals(trim($outputString), trim($this->p->parse($inputString)));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testListJustificationDoesntWork()
    {
        $inputString = "# some <right>justified</right> text\n# some more text\n";
        $outputString = "<document><list><ol><li><![CDATA[some ]]><right><origin><![CDATA[html]]></origin><![CDATA[justified]]></right><![CDATA[ text]]></li><li><![CDATA[some more text]]></li></ol></list></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testListLineBreak()
    {
        $inputString = "# some <br/> text\n# broken\n";
        $outputString = "<document><list><ol><li><![CDATA[some ]]><br/><![CDATA[ text]]></li><li><![CDATA[broken]]></li></ol></list></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testListLink()
    {
        $inputString = "# some ((Web|Ref)(Alias)) text\n# continues\n";
        $outputString = "<document><list><ol><li><![CDATA[some ]]><link><origin><![CDATA[round]]></origin><strref><![CDATA[WebRef]]></strref><![CDATA[Alias]]></link><![CDATA[ text]]></li><li><![CDATA[continues]]></li></ol></list></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testListURI()
    {
        $inputString = "# some http://www.h2g2.com\n# text\n";
        $outputString = "<document><list><ol><li><![CDATA[some ]]><uri><strref><![CDATA[http://www.h2g2.com]]></strref></uri></li><li><![CDATA[text]]></li></ol></list></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testListPlugin()
    {
        $inputString = "# text <plugin Example>\n";
        $outputString = "<document><list><ol><li><![CDATA[text ]]><plugin><name><![CDATA[Example]]></name></plugin></li></ol></list></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }

    //------------------------------------------------------------------
    //  Table of Contents
    //------------------------------------------------------------------
    function testTOC()
    {
        $inputString = "+ Heading1\nsome text\n<toc>";
        $outputString = "<h1><![CDATA[Heading1]]></h1><![CDATA[\n]]><![CDATA[some text\n]]><toc><ul><li><link><topicref><![CDATA[1]]></topicref><![CDATA[Heading1]]></link></li></ul></toc>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testTOC2()
    {
        $inputString = "+ ((http://archie.cowiki.org)(Alias))\nsome text\n<toc>";
        $outputString = "<h1><link><origin><![CDATA[round]]></origin><href><![CDATA[http://archie.cowiki.org]]></href><![CDATA[Alias]]></link></h1><![CDATA[\n]]><![CDATA[some text\n]]><toc><ul><li><link><topicref><![CDATA[1]]></topicref><![CDATA[Alias]]></link></li></ul></toc>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    //------------------------------------------------------------------
    //  emphasis spans links
    //------------------------------------------------------------------
    function testLinkSpan()
    {
        $inputString = "**Le portail internet des socits caldoniennes ((XL Programmation Micro)) et ((Pecsi)).**";
        $outputString = "<document><b><origin><![CDATA[wiki]]></origin><![CDATA[Le portail internet des socits caldoniennes ]]><link><origin><![CDATA[round]]></origin><strref><![CDATA[XL Programmation Micro]]></strref><![CDATA[]]></link><![CDATA[ et ]]><link><origin><![CDATA[round]]></origin><strref><![CDATA[Pecsi]]></strref><![CDATA[]]></link><![CDATA[.]]></b></document>";

        $this->assertEquals($outputString,$this->p->parse($inputString));
        $this->assertEquals($inputString,$this->r->parse($outputString));
    }
}

//--------------------------------------------------------------------------
//      Basic tests for formatting text within paragraphs
//--------------------------------------------------------------------------
class ParserTestTest extends PHPUnit_TestCase
{
    var $p = NULL;
    var $r = NULL;
    var $c = NULL;
    //--------------------------------------------------------------
    //  Standard constructor for unit tests.
    //--------------------------------------------------------------
    function __construct($name)
    {
        $this->PHPUnit_TestCase($name);
    }

    //--------------------------------------------------------------
    //  Setup: Set up an instance of the parser for testing
    //--------------------------------------------------------------
    function setUp()
    {
        $this->p = TestWikiParser :: getInstance();
        $this->r = TestReverseParser :: getInstance();
        $this->c = WikiPseudoXmlConverter :: getInstance();
    }

    function testCharacters() {

$sStr = <<<EOF
**Le portail internet des socits caldoniennes ((XL Programmation Micro)) et ((Pecsi)).**
EOF;

        $this->assertEquals($this->r->parse($this->p->parse($sStr)),$sStr);
}

    /*
<plugin name="doc.recent" limit="12" cutoff="30" title="Recently changed documents" style="float:right; margin: 0px 0px 5px 15px;"/>
<table attribute="value"><tr valign="top"><td colspan="1"> <em><strong>2 Feb 2005</strong></em> </td><td colspan="1"> <plugin name="embed" src="embed/news_cowiki.gif"/></td></tr></table>
<q><p>After the transfer of the coWiki project to the new maintainer Paul Hanchett, the new <link href="http://www.cowiki.org">site</link> is up and running. Latest versions and updates are available there. I'll successively close most coWiki related pages here on develnet.org.</p></q>

    */
    function testCharacters2() {
      $xml = "<document><link><strref><![CDATA[Hakan Kkyilmaz]]></strref><![CDATA[Alias]]></link></document>";
      $sStr = "((Hakan Kkyilmaz)(Alias))";
      $this->assertEquals($this->r->parse($xml),$sStr);
    }
}
//--------------------------------------------------------------------------
//      Basic tests for formatting text within paragraphs
//--------------------------------------------------------------------------
class ParserBlockFormattingTest extends PHPUnit_TestCase
{
    var $p = NULL;
    var $r = NULL;
    var $c = NULL;
    //--------------------------------------------------------------
    //  Standard constructor for unit tests.
    //--------------------------------------------------------------
    function __construct($name)
    {
        $this->PHPUnit_TestCase($name);
    }

    //--------------------------------------------------------------
    //  Setup: Set up an instance of the parser for testing
    //--------------------------------------------------------------
    function setUp()
    {
        $this->p = TestWikiParser :: getInstance();
        $this->r = TestReverseParser :: getInstance();
        $this->c = WikiPseudoXmlConverter :: getInstance();
    }

    /*
    function testBlock() {
        $inputString = "<q>\n quote test with <noop></q></noop>\n</q>";
        $outputString = "<document><q><![CDATA[\n]]><![CDATA[ quote test with ]]><noop><![CDATA[&lt;/q&gt;]]></noop><![CDATA[\n]]></q></document>";
        $this->assertEquals($outputString, $this->p->parse($inputString));
        $this->assertEquals($inputString, $this->r->parse($outputString));
    }
    function testBlock2() {
        $inputString = "12345678901234567890123456789012345678901234567890";
        //$outputString = "<document><q><![CDATA[\n]]><![CDATA[ quote test with ]]><noop><![CDATA[&lt;/q&gt;]]></noop><![CDATA[\n]]></q></document>";
        //$this->assertEquals($outputString, $this->p->parse($inputString));
        //$this->assertEquals($inputString, $this->r->parse($outputString));
        echo $this->p->parse($inputString);
    }
    */
    function testBlock() {
$sStr = <<<EOF
+ %TITLE%

**Le portail internet des socits caldoniennes ((XL Programmation Micro)) et ((Pecsi)).**

<q>
Ceci est donc notre wiki. Son objectif est videmment d'y mettre en commun toute la documentation, les howtos et outils ncssaire  notre travail quotidien.
</q>

<table>
| <plugin Embed src="/img/world.png" alt="Le Monde">
| <plugin Embed src="/img/nc.png" alt="Nouvelle-Caledonie">
</table>

EOF;
        $xml = $this->p->parse($sStr);
        $text = $this->r->parse($xml);
        $this->assertEquals($text,$sStr);
    }
}
?>
